/*************************************************************/
/** GrayscaleLib - grayscale graphic library for fx-9860G   **/
/** GrayscaleLib is free software                           **/
/**                                                         **/
/** @author Pierre "PierrotLL" Le Gall                      **/
/** @contact legallpierre89@gmail.com                       **/
/** @author Florian "Eiyeron" Dormont                       **/
/** @contact Eiyeron@gmail.com                              **/
/**                                                         **/
/** @file GrayscaleLib.c                                    **/
/** Code file of GrayscaleLib                               **/
/**                                                         **/
/** @date 05-11-2012                                        **/
/*************************************************************/

#include "GrayscaleLib.h"
#include <stdlib.h>

/******************************/
/** Dependencies management  **/
/******************************/

#ifdef GL_ALL
#define GL_ENGINE
#define GL_CLEAR_BUFFERS
#define GL_CLEAR_SCREEN
#define GL_DISPLAY_BUFFERS
#define GL_SET_CONTRAST
#define GL_GET_CONTRAST
#define GL_PIXEL
#define GL_POINT
#define GL_PIXEL_TEST
#define GL_LINE
#define GL_HORIZONTAL_LINE
#define GL_VERTICAL_LINE
#define GL_RECTANGLE
#define GL_POLYGONE
#define GL_FILLED_POLYGONE
#define GL_CIRCLE
#define GL_FILLED_CIRCLE
#define GL_ELLIPSE
#define GL_ELLIPSE_IN_RECT
#define GL_FILLED_ELLIPSE
#define GL_FILLED_ELLIPSE_IN_RECT
#define GL_HORIZONTAL_SCROLL
#define GL_VERTICAL_SCROLL
#define GL_BMP_OR
#define GL_BMP_AND
#define GL_BMP_XOR
#define GL_BMP_OR_CL
#define GL_BMP_AND_CL
#define GL_BMP_XOR_CL
#define GL_BMP_8_OR
#define GL_BMP_8_AND
#define GL_BMP_8_XOR
#define GL_BMP_8_OR_CL
#define GL_BMP_8_AND_CL
#define GL_BMP_8_XOR_CL
#define GL_BMP_16_OR
#define GL_BMP_16_AND
#define GL_BMP_16_XOR
#define GL_BMP_16_OR_CL
#define GL_BMP_16_AND_CL
#define GL_BMP_16_XOR_CL
#endif

#ifdef GL_POLYGONE
#define GL_LINE
#endif

#ifdef GL_LINE
#define GL_PIXEL
#endif

#ifdef GL_POINT
#define GL_PIXEL
#define GL_RECTANGLE
#endif

#ifdef GL_RECTANGLE
#define GL_HORIZONTAL_LINE
#endif

#ifdef GL_FILLED_POLYGONE
#define GL_HORIZONTAL_LINE
#endif

#ifdef GL_CIRCLE
#define GL_PIXEL
#endif

#ifdef GL_FILLED_CIRCLE
#define GL_HORIZONTAL_LINE
#endif

#ifdef GL_ELLIPSE_IN_RECT
#define GL_ELLIPSE
#endif

#ifdef GL_ELLIPSE
#define GL_PIXEL
#endif

#ifdef GL_FILLED_ELLIPSE_IN_RECT
#define GL_FILLED_ELLIPSE
#endif

#ifdef GL_FILLED_ELLIPSE
#define GL_HORIZONTAL_LINE
#endif

Screen* screen = NULL;

/***************/
/** Functions **/
/***************/

#define sgn(x)	(x<0?-1:1)
#define rnd(x)	((int)(x+0.5))


/**********************/
/* Graphics Functions */
/**********************/
char* GL_vram_adress(Buffer choice)
{
	if(choice == LIGHT_BUFFER)
		return screen->VRAM1;
	else
		return screen->VRAM2;
}

#ifdef GL_CLEAR_BUFFERS
void GL_clear_buffers(Buffer choice)
{
	int i, end, *pointer_long, vram;
	char *pointer_byte;
	if(choice== BOTH_BUFFERS) {
		GL_clear_buffers(LIGHT_BUFFER);
		GL_clear_buffers(DARK_BUFFER);
		return;
	}	
	vram = (int) GL_vram_adress(choice);
	end = 4-vram&3;
	pointer_byte = (char*)vram;
	for(i=0 ; i<end ; i++) pointer_byte[i] = 0;
	pointer_long = (int*) (vram+end);
	for(i=0 ; i<255 ; i++) pointer_long[i] = 0;
	pointer_byte += 1020+end;
	end = vram&3;
	for(i=0 ; i<end ; i++) pointer_byte[i] = 0;
}
#endif

#ifdef GL_CLEAR_SCREEN
void GL_clear_screen()
{
	char *LCD_register_selector = (char*)0xB4000000, *LCD_data_register = (char*)0xB4010000;
	int i, j;
	for(i=0 ; i<64 ; i++) {
		*LCD_register_selector = 4;
		*LCD_data_register = i|192;
		*LCD_register_selector = 4;
		*LCD_data_register = 0;
		*LCD_register_selector = 7;
		for(j=0 ; j<16 ; j++) *LCD_data_register = 0;
	}
}
#endif


#ifdef GL_DISPLAY_BUFFERS
void GL_display_buffers(Buffer choice)
{
	if(choice == BOTH_BUFFERS) {
		GL_display_buffers(LIGHT_BUFFER);
		GL_display_buffers(DARK_BUFFER);
		return;
	}	
	char *buffer_selector = choice == LIGHT_BUFFER? screen->DD1 : screen->DD2;
	char *LCD_data_register = (char*)0xB4010000, *vram;
	int i, j;
	vram = GL_vram_adress(choice);
	for(i=0 ; i<64 ; i++) {
		*LCD_register_selector = 4;
		*LCD_data_register = i|192;
		*LCD_register_selector = 4;
		*LCD_data_register = 0;
		*LCD_register_selector = 7;
		for(j=0 ; j<16 ; j++) *LCD_data_register = *vram++;
	}
}
#endif

#ifdef GL_SET_CONTRAST
void GL_set_contrast(unsigned char contrast)
{
	char *LCD_register_selector = (char*)0xB4000000, *LCD_data_register = (char*)0xB4010000;
	*LCD_register_selector = 6;
	*LCD_data_register = contrast;
}
#endif

#ifdef GL_GET_CONTRAST
unsigned char GL_get_contrast()
{
	char *LCD_register_selector = (char*)0xB4000000, *LCD_data_register = (char*)0xB4010000;
	*LCD_register_selector = 6;
	return *LCD_data_register;
}
#endif

#ifdef GL_PIXEL
void GL_pixel(int x, int y, GL_Color color)
{
	if(x<0 || x>127 || y<0 || y>63) return;
	switch(color)
	{
		case WHITE:
			screen->VRAM1[(y<<4)+(x>>3)] &= ~(128>>(x&7));
			screen->VRAM2[(y<<4)+(x>>3)] &= ~(128>>(x&7));
			break;
		case LIGHT_GRAY:
			screen->VRAM1[(y<<4)+(x>>3)] |= (128>>(x&7));
			screen->VRAM2[(y<<4)+(x>>3)] &= ~(128>>(x&7));
			break;
		case DARK_GRAY:
			screen->VRAM1[(y<<4)+(x>>3)] &= ~(128>>(x&7));
			screen->VRAM1[(y<<4)+(x>>3)] |= (128>>(x&7));
			break;
		case BLACK:
			screen->VRAM1[(y<<4)+(x>>3)] |= (128>>(x&7));
			screen->VRAM2[(y<<4)+(x>>3)] |= (128>>(x&7));
			break;
		case XOR:
			screen->VRAM1[(y<<4)+(x>>3)] ^= 128>>(x&7);
			screen->VRAM2[(y<<4)+(x>>3)] ^= 128>>(x&7);
			break;
	}
}
#endif

#ifdef GL_POINT
void GL_point(int x, int y, int width, GL_Color color)
{
	if(width < 1) return;
	if(width == 1) GL_pixel(x, y, color);
	else
	{
		int padding, pair;
		padding = width>>1;
		pair = !(width&1);
		GL_rectangle(x-padding+pair, y-padding+pair, x+padding, y+padding, 0, 0, color);
	}
}
#endif

#ifdef GL_PIXEL_TEST
GL_Color GL_pixel_test(int x, int y)
{
	char byte, result = 0;
	if(x<0 || x>127 || y<0 || y>63) return TRANSPARENT;
	byte = 1<<(7-(x&7));
	result = screen->VRAM1[(y<<4)+(x>>3)] & byte) + 2*screen->VRAM2[(y<<4)+(x>>3)] & byte);
	switch(result) {
	case 0:
		return WHITE;
		break;
	case 1:
		return LIGHT_GRAY;
		break;
	case 2:
		return DARK_GRAY;
		break;
	case 3:
		return BLACK;
		break;
	}

}
#endif

#ifdef GL_LINE
void GL_line(int x1, int y1, int x2, int y2, GL_Color color)
{
	int i, x, y, dx, dy, sx, sy, cumul;
	x = x1;
	y = y1;
	dx = x2 - x1;
	dy = y2 - y1;
	sx = sgn(dx);
	sy = sgn(dy);
	dx = abs(dx);
	dy = abs(dy);
	GL_pixel(x, y, color);
	if(dx > dy)
	{
		cumul = dx / 2;
		for(i=1 ; i<dx ; i++)
		{
			x += sx;
			cumul += dy;
			if(cumul > dx)
			{
				cumul -= dx;
				y += sy;
			}
			GL_pixel(x, y, color);
		}
	}
	else
	{
		cumul = dy / 2;
		for(i=1 ; i<dy ; i++)
		{
			y += sy;
			cumul += dx;
			if(cumul > dy)
			{
				cumul -= dy;
				x += sx;
			}
			GL_pixel(x, y, color);
		}
	}
}
#endif

#ifdef GL_HORIZONTAL_LINE
void GL_horizontal_line(int y, int x1, int x2, GL_Color color)
{
    int i;
    if(y<0 || y>63 || (x1<0 && x2<0) || (x1>127 && x2>127)) return;
    if(x1 > x2)
    {
    	i = x1;
    	x1 = x2;
    	x2 = i;
    }
    if(x1 < 0) x1 = 0;
    if(x2 > 127) x2 = 127;
    switch(color)
    {
    	case BLACK:
			if(x1>>3 != x2>>3)
			{
				screen->VRAM1[(y<<4)+(x1>>3)] |= 255 >> (x1&7);
				screen->VRAM2[(y<<4)+(x1>>3)] |= 255 >> (x1&7);
				screen->VRAM1[(y<<4)+(x2>>3)] |= 255 << 7-(x2&7);
				screen->VRAM2[(y<<4)+(x2>>3)] |= 255 << 7-(x2&7);
				for(i=(x1>>3)+1 ; i<x2>>3 ; i++)
				{
					screen->VRAM1[(y<<4) + i] = 255;
					screen->VRAM2[(y<<4) + i] = 255;
				}
			}
			else
			{			
				screen->VRAM1[(y<<4)+(x1>>3)] |= (255>>(x1%8 + 7-x2%8))<<(7-(x2&7));
				screen->VRAM2[(y<<4)+(x1>>3)] |= (255>>(x1%8 + 7-x2%8))<<(7-(x2&7));
			}
			break;
    	case DARK_GRAY:
			if(x1>>3 != x2>>3)
			{
				screen->VRAM1[(y<<4)+(x1>>3)] |= 255 >> (x1&7);
				screen->VRAM2[(y<<4)+(x1>>3)] &= 255 << 8-(x1&7);
				screen->VRAM1[(y<<4)+(x2>>3)] |= 255 << 7-(x2&7);
				screen->VRAM2[(y<<4)+(x2>>3)] &= 255 >> 1+(x2&7);
				for(i=(x1>>3)+1 ; i<x2>>3 ; i++)
				{
					screen->VRAM1[(y<<4) + i] = 0;
					screen->VRAM2[(y<<4) + i] = 255;
				}
			}
			else
			{			
				screen->VRAM1[(y<<4)+(x1>>3)] |= (255>>(x1%8 + 7-x2%8))<<(7-(x2&7));
				screen->VRAM2[(y<<4)+(x1>>3)] &= (255<<8-(x1&7)) | (255>>1+(x2&7));
			}
			break;
    	case LIGHT_GRAY:
			if(x1>>3 != x2>>3)
			{
				screen->VRAM1[(y<<4)+(x1>>3)] &= 255 << 8-(x1&7);
				screen->VRAM2[(y<<4)+(x1>>3)] |= 255 >> (x1&7);
				screen->VRAM1[(y<<4)+(x2>>3)] &= 255 >> 1+(x2&7);
				screen->VRAM2[(y<<4)+(x2>>3)] |= 255 << 7-(x2&7);
				for(i=(x1>>3)+1 ; i<x2>>3 ; i++)
				{
					screen->VRAM1[(y<<4) + i] = 255;
					screen->VRAM2[(y<<4) + i] = 0;
				}
			}
			else
			{			
				screen->VRAM1[(y<<4)+(x1>>3)] &= (255<<8-(x1&7)) | (255>>1+(x2&7));
				screen->VRAM2[(y<<4)+(x1>>3)] |= (255>>(x1%8 + 7-x2%8))<<(7-(x2&7));
			}
			break;
		case WHITE:
			if(x1>>3 != x2>>3)
			{
				screen->VRAM1[(y<<4)+(x1>>3)] &= 255 << 8-(x1&7);
				screen->VRAM2[(y<<4)+(x1>>3)] &= 255 << 8-(x1&7);
				screen->VRAM1[(y<<4)+(x2>>3)] &= 255 >> 1+(x2&7);
				screen->VRAM2[(y<<4)+(x2>>3)] &= 255 >> 1+(x2&7);
				for(i=(x1>>3)+1 ; i<x2>>3 ; i++)
				{
					screen->VRAM1[(y<<4) + i] = 0;
					screen->VRAM2[(y<<4) + i] = 0;
				}
			}
			else 
			{
				screen->VRAM1[(y<<4)+(x1>>3)] &= (255<<8-(x1&7)) | (255>>1+(x2&7));
				screen->VRAM2[(y<<4)+(x1>>3)] &= (255<<8-(x1&7)) | (255>>1+(x2&7));
			}
			break;
		case XOR:
			if(x1>>3 != x2>>3)
			{
				screen->VRAM1[(y<<4)+(x1>>3)] ^= 255 >> (x1&7);
				screen->VRAM2[(y<<4)+(x1>>3)] ^= 255 >> (x1&7);
				screen->VRAM1[(y<<4)+(x2>>3)] ^= 255 << 7-(x2&7);
				screen->VRAM2[(y<<4)+(x2>>3)] ^= 255 << 7-(x2&7);
				for(i=(x1>>3)+1 ; i<(x2>>3) ; i++)
				{
					screen->VRAM1[(y<<4) + i] ^= 255;
					screen->VRAM2[(y<<4) + i] ^= 255;
				
				}
			}
			else
			{
			screen->VRAM1[(y<<4)+(x1>>3)] ^= (255>>((x1&7) + 7-(x2&7)))<<(7-(x2&7));
			screen->VRAM2[(y<<4)+(x1>>3)] ^= (255>>((x1&7) + 7-(x2&7)))<<(7-(x2&7));
			}
			break;

    }
}
#endif

#ifdef GL_VERTICAL_LINE
void GL_vertical_line(int x, int y1, int y2, GL_Color color)
{
    int i, j;
    char byte;
	if(x<0 || x>127 || (y1<0 && y2<0) || (y1>63 && y2>63)) return;
	if(y1 > y2)
	{
		int tmp = y1;
		y1 = y2;
		y2 = tmp;
	}
	if(y1 < 0) y1 = 0;
	if(y2 > 63) y2 = 63;

	i = (y1<<4)+(x>>3);
	j = (y2<<4)+(x>>3);
	switch(color)
	{
		case BLACK:
			byte = 128>>(x&7);
			for( ; i<=j ; i+=16)
			{
				screen->VRAM1[i] |= byte;
				screen->VRAM2[i] |= byte;
			}
			break;
		case DARK_GRAY:
			byte = 128>>(x&7);
			for( ; i<=j ; i+=16)
			 {
				screen->VRAM1[i] |= byte;
				screen->VRAM2[i] &= ~byte;			 
			 }
			break;
		case LIGHT_GRAY:
			byte = 128>>(x&7);
			for( ; i<=j ; i+=16)
			{
				screen->VRAM1[i] &= ~byte;			 
				screen->VRAM2[i] |= byte;
			}
			break;
		case WHITE:
			byte = ~(128>>(x&7));
			for( ; i<=j ; i+=16)
			{
				screen->VRAM1[i] &= byte;
				screen->VRAM2[i] &= byte;
			}
			break;
		case XOR:
			byte = 128>>(x&7);
			for( ; i<=j ; i+=16)
			{
			screen->VRAM1[i] ^= byte;
			screen->VRAM2[i] ^= byte;
			}
			break;

	}
}
#endif

#ifdef GL_RECTANGLE
void GL_rectangle(int x1, int y1, int x2, int y2, int border_width, GL_Color border_color, GL_Color fill_color)
{
	int i;
	if(x1 > x2)
	{
		i = x1;
		x1 = x2;
		x2 = i;
	}
	if(y1 > y2)
	{
		i = y1;
		y1 = y2;
		y2 = i;
	}
	if(border_width > (x2-x1)/2+1) border_width = (x2-x1)/2+1;
	if(border_width > (y2-y1)/2+1) border_width = (y2-y1)/2+1;
	if(border_color != TRANSPARENT && border_width > 0)
	{
		for(i=0 ; i<border_width ; i++)
		{
			GL_horizontal_line(y1+i, x1, x2, border_color);
			GL_horizontal_line(y2-i, x1, x2, border_color);
		}
		for(i=y1+border_width ; i<=y2-border_width ; i++)
		{
			GL_horizontal_line(i, x1, x1+border_width-1, border_color);
			GL_horizontal_line(i, x2-border_width+1, x2, border_color);
		}
	}
	if(fill_color != TRANSPARENT)
	{
		for(i=y1+border_width ; i<=y2-border_width ; i++)
			GL_horizontal_line(i, x1+border_width, x2-border_width, fill_color);
	}
}
#endif

#ifdef GL_POLYGONE
void GL_polygone(int *x, int *y, int nb_vertices, GL_Color color)
{
	int i;
	if(nb_vertices < 1) return;
	for(i=0 ; i<nb_vertices-1 ; i++)
		GL_line(x[i], y[i], x[i+1], y[i+1], color);
	GL_line(x[i], y[i], x[0], y[0], color);
}
#endif

#ifdef GL_FILLED_POLYGONE
int GL_filled_polygone_quicksord_partition(int *t, int p, int r)
{
	int i, j, x, tmp;
	j = p - 1;
	x = t[r];
	for(i=p ; i<r ; i++)
	{
		if(x > t[i])
		{
			j++;
			tmp = t[j];
			t[j] = t[i];
			t[i] = tmp;
		}
	}
	t[r] = t[j+1];
	t[j+1] = x;
	return j + 1;
}

void GL_filled_polygone_quicksord(int* t, int p, int r)
{
	int q;
	if(p < r)
	{
		q = GL_filled_polygone_quicksord_partition(t, p, r);
		GL_filled_polygone_quicksord(t, p, q-1);
		GL_filled_polygone_quicksord(t, q+1, r);
	}
}

void GL_filled_polygone(int *x, int *y, int nb_vertices, GL_Color color)
{
	int i, j, dx, dy, ymin, ymax;
	int *cut_in_line, nb_cut;
	if(nb_vertices < 3) return;
	cut_in_line = malloc(nb_vertices*sizeof(int));
	if(!cut_in_line) return;
	ymin = ymax = y[0];
	for(i=1 ; i<nb_vertices ; i++)
	{
		if(y[i] < ymin) ymin = y[i];
		if(y[i] > ymax) ymax = y[i];
	}
	for(i=ymin ; i<=ymax ; i++)
	{
		nb_cut = 0;
		for(j=0 ; j<nb_vertices ; j++)
		{
			if((y[j]<=i && y[(j+1)%nb_vertices]>=i) || (y[j]>=i && y[(j+1)%nb_vertices]<=i))
			{
				dy = abs(y[j]-y[(j+1)%nb_vertices]);
				if(dy)
				{
					dx = x[(j+1)%nb_vertices]-x[j];
					cut_in_line[nb_cut] = x[j] + rnd(abs(i-y[j]+sgn(i-y[j])/2)*dx/dy);
					nb_cut++;
				}
			}
		}
		GL_filled_polygone_quicksord(cut_in_line, 0, nb_cut-1);
		j = 0;
		while(j<nb_cut-2 && cut_in_line[j]==cut_in_line[j+1]) j++;
		while(j < nb_cut)
		{
			if(j == nb_cut-1) GL_horizontal_line(i, cut_in_line[j-1]+1, cut_in_line[j], color);
			else
			{
				dx = 1;
				while(j+dx<nb_cut-1 && cut_in_line[j+dx]==cut_in_line[j+dx+1]) dx++;
				GL_horizontal_line(i, cut_in_line[j], cut_in_line[j+dx], color);
				j += dx;
			}
			j++;
		}
	}
	free(cut_in_line);
}
#endif

#ifdef GL_CIRCLE
void GL_circle(int x, int y, int radius, GL_Color color)
{
	int plot_x, plot_y, d;

	if(radius < 0) return;
	plot_x = 0;
	plot_y = radius;
	d = 1 - radius;

	GL_pixel(x, y+plot_y, color);
	if(radius)
	{
		GL_pixel(x, y-plot_y, color);
		GL_pixel(x+plot_y, y, color);
		GL_pixel(x-plot_y, y, color);
	}
	while(plot_y > plot_x)
	{
		if(d < 0)
			d += 2*plot_x+3;
		else
		{
			d += 2*(plot_x-plot_y)+5;
			plot_y--;
		}
		plot_x++;
		if(plot_y >= plot_x)
		{
			GL_pixel(x+plot_x, y+plot_y, color);
			GL_pixel(x-plot_x, y+plot_y, color);
			GL_pixel(x+plot_x, y-plot_y, color);
			GL_pixel(x-plot_x, y-plot_y, color);
		}
		if(plot_y > plot_x)
		{
			GL_pixel(x+plot_y, y+plot_x, color);
			GL_pixel(x-plot_y, y+plot_x, color);
			GL_pixel(x+plot_y, y-plot_x, color);
			GL_pixel(x-plot_y, y-plot_x, color);
		}
	}
}
#endif

#ifdef GL_FILLED_CIRCLE
void GL_filled_circle(int x, int y, int radius, GL_Color color)
{
	int plot_x, plot_y, d;

	if(radius < 0) return;
	plot_x = 0;
	plot_y = radius;
	d = 1 - radius;

	GL_horizontal_line(y, x-plot_y, x+plot_y, color);
	while(plot_y > plot_x)
	{
		if(d < 0)
			d += 2*plot_x+3;
		else {
			d += 2*(plot_x-plot_y)+5;
			plot_y--;
			GL_horizontal_line(y+plot_y+1, x-plot_x, x+plot_x, color);
			GL_horizontal_line(y-plot_y-1, x-plot_x, x+plot_x, color);
		}
		plot_x++;
		if(plot_y >= plot_x)
		{
			GL_horizontal_line(y+plot_x, x-plot_y, x+plot_y, color);
			GL_horizontal_line(y-plot_x, x-plot_y, x+plot_y, color);
		}
	}
}
#endif

#ifdef GL_ELLIPSE
void GL_ellipse(int x, int y, int radius1, int radius2, GL_Color color)
{
	int plot_x, plot_y;
	float d1, d2;
	if(radius1 < 1 || radius2 < 1) return;
	plot_x = 0;
	plot_y = radius2;
	d1 = radius2*radius2 - radius1*radius1*radius2 + radius1*radius1/4;
	GL_pixel(x, y+plot_y, color);
	GL_pixel(x, y-plot_y, color);
	while(radius1*radius1*(plot_y-.5) > radius2*radius2*(plot_x+1))
	{
		if(d1 < 0)
		{
			d1 += radius2*radius2*(2*plot_x+3);
			plot_x++;
		} else {
			d1 += radius2*radius2*(2*plot_x+3) + radius1*radius1*(-2*plot_y+2);
			plot_x++;
			plot_y--;
		}
		GL_pixel(x+plot_x, y+plot_y, color);
		GL_pixel(x-plot_x, y+plot_y, color);
		GL_pixel(x+plot_x, y-plot_y, color);
		GL_pixel(x-plot_x, y-plot_y, color);
	}
	d2 = radius2*radius2*(plot_x+.5)*(plot_x+.5) + radius1*radius1*(plot_y-1)*(plot_y-1) - radius1*radius1*radius2*radius2;
	while(plot_y > 0)
	{
		if(d2 < 0)
		{
			d2 += radius2*radius2*(2*plot_x+2) + radius1*radius1*(-2*plot_y+3);
			plot_y--;
			plot_x++;
		} else {
			d2 += radius1*radius1*(-2*plot_y+3);
			plot_y--;
		}
		GL_pixel(x+plot_x, y+plot_y, color);
		GL_pixel(x-plot_x, y+plot_y, color);
		if(plot_y > 0)
		{
			GL_pixel(x+plot_x, y-plot_y, color);
			GL_pixel(x-plot_x, y-plot_y, color);
		}
	}
}
#endif

#ifdef GL_ELLIPSE_IN_RECT
void GL_ellipse_in_rect(int x1, int y1, int x2, int y2, GL_Color color)
{
	int radius1, radius2;
	if(x1 > x2)
	{
		int tmp = x1;
		x1 = x2;
		x2 = tmp;
	}
	if(y1 > y2)
	{
		int tmp = y1;
		y1 = y2;
		y2 = tmp;
	}
	radius1 = (x2-x1)/2;
	radius2 = (y2-y1)/2;
	GL_ellipse(x1+radius1, y1+radius2, radius1, radius2, color);
}
#endif

#ifdef GL_FILLED_ELLIPSE
void GL_filled_ellipse(int x, int y, int radius1, int radius2, GL_Color color)
{
	int plot_x, plot_y;
	float d1, d2;
	if(radius1 < 1 || radius2 < 1) return;
	plot_x = 0;
	plot_y = radius2;
	d1 = radius2*radius2 - radius1*radius1*radius2 + radius1*radius1/4;
	while(radius1*radius1*(plot_y-.5) > radius2*radius2*(plot_x+1))
	{
		if(d1 < 0)
		{
			d1 += radius2*radius2*(2*plot_x+3);
			plot_x++;
		} else {
			d1 += radius2*radius2*(2*plot_x+3) + radius1*radius1*(-2*plot_y+2);
			GL_horizontal_line(y+plot_y, x-plot_x, x+plot_x, color);
			GL_horizontal_line(y-plot_y, x-plot_x, x+plot_x, color);
			plot_x++;
			plot_y--;
		}
	}
	GL_horizontal_line(y+plot_y, x-plot_x, x+plot_x, color);
	GL_horizontal_line(y-plot_y, x-plot_x, x+plot_x, color);
	d2 = radius2*radius2*(plot_x+.5)*(plot_x+.5) + radius1*radius1*(plot_y-1)*(plot_y-1) - radius1*radius1*radius2*radius2;
	while(plot_y > 0)
	{
		if(d2 < 0)
		{
			d2 += radius2*radius2*(2*plot_x+2) + radius1*radius1*(-2*plot_y+3);
			plot_y--;
			plot_x++;
		} else {
			d2 += radius1*radius1*(-2*plot_y+3);
			plot_y--;
		}
		GL_horizontal_line(y+plot_y, x-plot_x, x+plot_x, color);
		if(plot_y > 0)
			GL_horizontal_line(y-plot_y, x-plot_x, x+plot_x, color);
	}
}
#endif

#ifdef GL_FILLED_ELLIPSE_IN_RECT
void GL_filled_ellipse_in_rect(int x1, int y1, int x2, int y2, GL_Color color)
{
	int radius1, radius2;
	if(x1 > x2)
	{
		int tmp = x1;
		x1 = x2;
		x2 = tmp;
	}
	if(y1 > y2)
	{
		int tmp = y1;
		y1 = y2;
		y2 = tmp;
	}
	radius1 = (x2-x1)/2;
	radius2 = (y2-y1)/2;
	GL_filled_ellipse(x1+radius1, y1+radius2, radius1, radius2, color);
}
#endif

#ifdef GL_HORIZONTAL_SCROLL
void GL_horizontal_scroll(int scroll, Buffer choice)
{
	int i, j;
	unsigned char line[16], Bshift, next, *vram;
	unsigned short word;
	if(choice == BOTH_BUFFERS)
	{
		GL_horizontal_scroll(scroll, LIGHT_BUFFER);
		GL_horizontal_scroll(scroll, DARK_BUFFER);
		return;
	}
	vram = GL_vram_adress(choice);
	scroll %= 128;
	Bshift = 8-(scroll&7);
	for(i=0 ; i<64 ; i++)
	{
		for(j=0 ; j<16 ; j++) line[j] = vram[(i<<4)+((j-(scroll>>3)+15)&15)];
		next = line[15];
		vram[(i<<4)+15] = 0;
		for(j=15 ; j>0 ; j--)
		{
			word = next << Bshift;
			next = line[j-1];
			vram[(i<<4)+j] |= *((char*)&word+1);
			vram[(i<<4)+j-1] = *((char*)&word);
		}
		word = next << Bshift;
		vram[(i<<4)] |= *((char*)&word+1);
		vram[(i<<4)+15] |= *((char*)&word);
	}
}
#endif

#ifdef GL_VERTICAL_SCROLL
void GL_vertical_scroll(int scroll, Buffer choice)
{
	int i, j;
	char column[64], *vram;
		if(choice == BOTH_BUFFERS)
	{
		GL_vertical_scroll(scroll, LIGHT_BUFFER);
		GL_vertical_scroll(scroll, DARK_BUFFER);
		return;
	}
	vram = GL_vram_adress(choice);
	scroll %= 64;
	for(i=0 ; i<16 ; i++)
	{
		for(j=0 ; j<64 ; j++) column[j] = vram[(j<<4)+i];
		for(j=0 ; j<64 ; j++) vram[(j<<4)+i] = column[(j-scroll+64)&63];
	}
}
#endif


/******************************/
/* Grayscale Engine Functions */
/******************************/

#ifdef GL_ENGINE
#define INT_OFFSET 0x600

#define set_vbr(base)		_builtin_set_vbr(base)
struct st_tmu {
			  union {
					unsigned char BYTE;
					struct {
						   unsigned char	 :5;
						   unsigned char STR2:1;
						   unsigned char STR1:1;
						   unsigned char STR0:1;
						   }	  BIT;
					}		   TSTR;
};
struct st_tmu0 {
			   unsigned int	 TCOR;
			   unsigned int	 TCNT;
			   union {
					 unsigned short WORD;
					 struct {
							unsigned short	 :7;
							unsigned short UNF :1;
							unsigned short	 :2;
							unsigned short UNIE:1;
							unsigned short CKEG:2;
							unsigned short TPSC:3;
							}	   BIT;
					 }		  TCR;
};
struct st_intc {
			   union {
					 unsigned short WORD;
					 struct {
							unsigned short NMIL:1;
							unsigned short	 :6;
							unsigned short NMIE:1;
							}	   BIT;
					 }		  ICR0;
			   union {
					 unsigned short WORD;
					 struct {
							unsigned short _TMU0:4;
							unsigned short _TMU1:4;
							unsigned short _TMU2:4;
							unsigned short _RTC :4;
							}	   BIT;
					 }		  IPRA;
			   union {
					 unsigned short WORD;
					 struct {
							unsigned short _WDT:4;
							unsigned short _REF:4;
							}	   BIT;
					 }		  IPRB;
			   char			 wk[234];
			   unsigned int	 TRA;
			   unsigned int	 EXPEVT;
			   unsigned int	 INTEVT;
};
struct st_intx {
			   unsigned int	 INTEVT2;
			   union {
					 unsigned char BYTE;
					 struct {
							unsigned char PINT0R:1;
							unsigned char PINT1R:1;
							unsigned char IRQ5R :1;
							unsigned char IRQ4R :1;
							unsigned char IRQ3R :1;
							unsigned char IRQ2R :1;
							unsigned char IRQ1R :1;
							unsigned char IRQ0R :1;
							}	  BIT;
					 }		  IRR0;
			   char			 wk1;
			   union {
					 unsigned char BYTE;
					 struct {
							unsigned char TXI0R:1;
							unsigned char	  :1;
							unsigned char RXI0R:1;
							unsigned char ERI0R:1;
							unsigned char DEI3R:1;
							unsigned char DEI2R:1;
							unsigned char DEI1R:1;
							unsigned char DEI0R:1;
							}	  BIT;
					 }		  IRR1;
			   char			 wk2;
			   union {
					 unsigned char BYTE;
					 struct {
							unsigned char	  :3;
							unsigned char ADIR :1;
							unsigned char TXI2R:1;
							unsigned char	  :1;
							unsigned char RXI2R:1;
							unsigned char ERI2R:1;
							}	  BIT;
					 }		  IRR2;
			   char			 wk3[7];
			   union {
					 unsigned short WORD;
					 struct {
							unsigned short MAI   :1;
							unsigned short IRQLVL:1;
							unsigned short BLMSK :1;
							unsigned short	   :1;
							unsigned short IRQ5S :2;
							unsigned short IRQ4S :2;
							unsigned short IRQ3S :2;
							unsigned short IRQ2S :2;
							unsigned short IRQ1S :2;
							unsigned short IRQ0S :2;
							}	   BIT;
					 }		  ICR1;
			   union {
					 unsigned short WORD;
					 struct {
							unsigned short PINT15S:1;
							unsigned short PINT14S:1;
							unsigned short PINT13S:1;
							unsigned short PINT12S:1;
							unsigned short PINT11S:1;
							unsigned short PINT10S:1;
							unsigned short PINT9S :1;
							unsigned short PINT8S :1;
							unsigned short PINT7S :1;
							unsigned short PINT6S :1;
							unsigned short PINT5S :1;
							unsigned short PINT4S :1;
							unsigned short PINT3S :1;
							unsigned short PINT2S :1;
							unsigned short PINT1S :1;
							unsigned short PINT0S :1;
							}	   BIT;
					 }		  ICR2;
			   union {
					 unsigned short WORD;
					 struct {
							unsigned short PINT15E:1;
							unsigned short PINT14E:1;
							unsigned short PINT13E:1;
							unsigned short PINT12E:1;
							unsigned short PINT11E:1;
							unsigned short PINT10E:1;
							unsigned short PINT9E :1;
							unsigned short PINT8E :1;
							unsigned short PINT7E :1;
							unsigned short PINT6E :1;
							unsigned short PINT5E :1;
							unsigned short PINT4E :1;
							unsigned short PINT3E :1;
							unsigned short PINT2E :1;
							unsigned short PINT1E :1;
							unsigned short PINT0E :1;
							}	   BIT;
					 }		  PINTER;
			   union {
					 unsigned short WORD;
					 struct {
							unsigned short _IRQ3:4;
							unsigned short _IRQ2:4;
							unsigned short _IRQ1:4;
							unsigned short _IRQ0:4;
							}	   BIT;
					 }		  IPRC;
			   union {
					 unsigned short WORD;
					 struct {
							unsigned short _PINT0_7 :4;
							unsigned short _PINT8_15:4;
							unsigned short _IRQ5	:4;
							unsigned short _IRQ4	:4;
							}	   BIT;
					 }		  IPRD;
			   union {
					 unsigned short WORD;
					 struct {
							unsigned short _DMAC :4;
							unsigned short _SCIF0:4;
							unsigned short _SCIF2:4;
							unsigned short _ADC  :4;
							}	   BIT;
					 }		  IPRE;
			   char			 wk4[524260];
			   union {
					 unsigned short WORD;
					 struct {
							unsigned short	 :8;
							unsigned short _USB:4;
							}	   BIT;
					 }		  IPRF;
			   union {
					 unsigned short WORD;
					 struct {
							unsigned short _TPU0:4;
							unsigned short _TPU1:4;
							}	   BIT;
					 }		  IPRG;
			   union {
					 unsigned short WORD;
					 struct {
							unsigned short _TPU2:4;
							unsigned short _TPU3:4;
							}	   BIT;
					 }		  IPRH;
};

#define INTC   (*(volatile struct st_intc  *)0xFFFFFEE0)
#define INTX   (*(volatile struct st_intx  *)0xA4000000)
#define TMU	(*(volatile struct st_tmu   *)0xFFFFFE92)
#define TMU0   (*(volatile struct st_tmu0  *)0xFFFFFE94)


void* oldvbr;
unsigned short rwa;
unsigned short rwd;

#pragma interrupt(GL_interrupt)

void Gl_interrupt()
{
int evt = INTC.INTEVT;

if(evt != 0x400) {

	TMU0.TCR.BIT.UNF = 0;
	return;
}
	TMU0.TCR.BIT.UNF = 0;
	if(screen->which == 1)
	{
		draw_all(screen->DD1);
		screen->which = 2;
		TMU0.TCNT = TMU0.TCOR = screen->time1;
	} else {
		draw_all(screen->DD2);
		screen->which = 1;
		TMU0.TCNT = TMU0.TCOR = screen->time2;
	}
	TMU.TSTR.BIT.STR0 = 1;
}

int GL_init()
{
	screen = malloc(sizeof(Screen));
	if(!screen) return 0;
	screen->VRAM1 = malloc(1024);
	screen->VRAM2 = malloc(1024);
	screen->DD1 = malloc(1024);
	screen->DD2 = malloc(1024);
	if(!screen->VRAM1 || !screen->VRAM2 || !screen->DD1 || !screen->DD2) return 0;
	screen->which = 1;
	oldvbr = _builtin_get_vbr();
	screen->time1 = 3215;
	screen->time2 = 6831;
	set_vbr((void *)((unsigned int)GL_interrupt - INT_OFFSET));
	TMU0.TCR.WORD = 0x0022;
	TMU0.TCNT = TMU0.TCOR = screen->time1;
	rwa = INTC.IPRA.WORD;
	rwd = INTX.IPRD.WORD;
	INTC.IPRA.WORD = 0xf000;
	INTX.IPRD.WORD = 0x0000;
	TMU.TSTR.BIT.STR0 = 1;
	return 1;
}

int GL_init_timers(int timer 1, int timer 2)
{
	Gl_init();
	screen->time1 = timer1;
	screen->time2 = timer2;
}

void GL_quit()
{
	TMU.TSTR.BIT.STR0 = 0;
	TMU0.TCR.WORD = 0x0000;
	TMU0.TCNT = TMU0.TCOR = 0xFFFFFFFF;
	set_vbr(oldvbr);
	INTC.IPRA.WORD = rwa;
	INTX.IPRD.WORD =rwd;
	free(screen->VRAM1);
	free(screen->VRAM2);
	free(screen->DD1);
	free(screen->DD2);
	free(screen);

}

Screen* GL_getScreen()
{
	return screen;
}
#endif

