/*************************************************************/
/** GrayscaleLib - grayscale graphic library for fx-9860G   **/
/** GrayscaleLib is free software                           **/
/**                                                         **/
/** @author Pierre "PierrotLL" Le Gall                      **/
/** @contact legallpierre89@gmail.com                       **/
/** @author Florian "Eiyeron" Dormont                       **/
/** @contact Eiyeron@gmail.com                              **/
/**                                                         **/
/** @file GrayscaleLib.h                                    **/
/** Include header for GrayscaleLib                        **/
/**                                                         **/
/** @date 05-11-2011                                        **/
/*************************************************************/

#ifndef GRAYSCALELIB
#define GRAYSCALELIB




/****************************************************/
/** uncomment #define of functions you want to use **/
/****************************************************/

// #define GL_ALL //Auto define all functions

// #define GL_ENGINE

// #define GL_CLEAR_BUFFERS
// #define GL_CLEAR_SCREEN
// #define GL_DISPLAY_BUFFERS

// #define GL_SET_CONTRAST
// #define GL_GET_CONTRAST

// #define GL_PIXEL
// #define GL_POINT
// #define GL_PIXEL_TEST

// #define GL_LINE
// #define GL_HORIZONTAL_LINE
// #define GL_VERTICAL_LINE

// #define GL_RECTANGLE

// #define GL_POLYGONE
// #define GL_FILLED_POLYGONE

// #define GL_CIRCLE
// #define GL_FILLED_CIRCLE

// #define GL_ELLIPSE
// #define GL_ELLIPSE_IN_RECT
// #define GL_FILLED_ELLIPSE
// #define GL_FILLED_ELLIPSE_IN_RECT

// #define GL_HORIZONTAL_SCROLL
// #define GL_VERTICAL_SCROLL

// #define GL_BMP_OR
// #define GL_BMP_AND
// #define GL_BMP_XOR
// #define GL_BMP_OR_CL
// #define GL_BMP_AND_CL
// #define GL_BMP_XOR_CL

// #define GL_BMP_8_OR
// #define GL_BMP_8_AND
// #define GL_BMP_8_XOR
// #define GL_BMP_8_OR_CL
// #define GL_BMP_8_AND_CL
// #define GL_BMP_8_XOR_CL

// #define GL_BMP_16_OR
// #define GL_BMP_16_AND
// #define GL_BMP_16_XOR
// #define GL_BMP_16_OR_CL
// #define GL_BMP_16_AND_CL
// #define GL_BMP_16_XOR_CL


/*********************************/
/** Grayscale engine prototypes **/
/*********************************/


typedef struct {
	char *DD1, *DD2, *VRAM1, *VRAM2, which;
	int time1, time2;
} Screen;


void GL_interrupt();
int GL_init();
int GL_init_timers(int timer1, int timer2);
void GL_quit();
Screen* GL_getScreen();

/**************************/
/** Functions prototypes **/
/**************************/

#ifdef __cplusplus
extern "C" {
#endif

#define CONTRAST_MIN	130
#define CONTRAST_NORMAL	168
#define CONTRAST_MAX	190
typedef enum {TRANSPARENT=-1, WHITE, BLACK, XOR, CHECKER} GL_Color;
typedef enum {LIGHT_BUFFER, DARK_BUFFER, BOTH_BUFFERS} Buffer;

char* GL_vram_adress();

void GL_clear_buffers();
void GL_clear_screen();
void GL_display_vram();

void GL_set_contrast(unsigned char contrast);
unsigned char GL_get_contrast();

void GL_pixel(int x, int y, GL_Color color);
void GL_point(int x, int y, int width, GL_Color color);
GL_Color GL_pixel_test(int x, int y);

void GL_line(int x1, int y1, int x2, int y2, GL_Color color);
void GL_horizontal_line(int y, int x1, int x2, GL_Color color);
void GL_vertical_line(int x, int y1, int y2, GL_Color color);

void GL_rectangle(int x1, int y1, int x2, int y2, int border_width, GL_Color border_color, GL_Color fill_color);

void GL_polygone(int *x, int *y, int nb_vertices, GL_Color color);
void GL_filled_polygone(int *x, int *y, int nb_vertices, GL_Color color);

void GL_circle(int x, int y, int radius, GL_Color color);
void GL_filled_circle(int x, int y, int radius, GL_Color color);

void GL_ellipse(int x, int y, int radius1, int radius2, GL_Color color);
void GL_ellipse_in_rect(int x1, int y1, int x2, int y2, GL_Color color);
void GL_filled_ellipse(int x, int y, int radius1, int radius2, GL_Color color);
void GL_filled_ellipse_in_rect(int x, int y, int radius1, int radius2, GL_Color color);

void GL_horizontal_scroll(int scroll);
void GL_vertical_scroll(int scroll);

/*
void GL_bmp_or(unsigned char *bmp, int x, int y, int width, int height);
void GL_bmp_and(unsigned char *bmp, int x, int y, int width, int height);
void GL_bmp_xor(unsigned char *bmp, int x, int y, int width, int height);
void GL_bmp_or_cl(unsigned char *bmp, int x, int y, int width, int height);
void GL_bmp_and_cl(unsigned char *bmp, int x, int y, int width, int height);
void GL_bmp_xor_cl(unsigned char *bmp, int x, int y, int width, int height);

void GL_bmp_8_or(unsigned char *bmp, int x, int y);
void GL_bmp_8_and(unsigned char *bmp, int x, int y);
void GL_bmp_8_xor(unsigned char *bmp, int x, int y);
void GL_bmp_8_or_cl(unsigned char *bmp, int x, int y);
void GL_bmp_8_and_cl(unsigned char *bmp, int x, int y);
void GL_bmp_8_xor_cl(unsigned char *bmp, int x, int y);

void GL_bmp_16_or(unsigned short *bmp, int x, int y);
void GL_bmp_16_and(unsigned short *bmp, int x, int y);
void GL_bmp_16_xor(unsigned short *bmp, int x, int y);
void GL_bmp_16_or_cl(unsigned short *bmp, int x, int y);
void GL_bmp_16_and_cl(unsigned short *bmp, int x, int y);
void GL_bmp_16_xor_cl(unsigned short *bmp, int x, int y);
*/ //WORK IN PROGRESS

#ifdef __cplusplus
}
#endif


#endif //GRAYSCALELIB
